@@ -4,10 +4,16 @@ from typing import Callable
 import numpy as np
 import numpy.typing as npt
 import pandas as pd
-import plotly.graph_objects as go
+
+try:
+    import plotly.graph_objects as go
+    from ipywidgets import widgets
+    from IPython.display import display
+except ImportError as e:
+    raise ImportError(
+        "This module requires plotly, ipywidgets and ipython to be installed."
+    ) from e
 from lmfit import minimize, Parameters
-from ipywidgets import widgets
-from IPython.display import display
 from sys import float_info
 from ..result import Result
 from ..utils import calc_pseudo_diel, calc_rho
@@ -18,7 +24,9 @@ from .decorator import FitDecorator
 class FitRho(FitDecorator):
     """A class to fit psi/delta or rho based ellipsometry data with two degress of freedom"""
 
-    def set_psi_delta(self, update_exp: bool = False, update_names: bool = False) -> None:
+    def set_psi_delta(
+        self, update_exp: bool = False, update_names: bool = False
+    ) -> None:
         """Sets Plot to Psi/Delta values
 
         Args:
@@ -33,14 +41,14 @@ class FitRho(FitDecorator):
         self.fig.data[3].y = data.delta
 
         if update_exp:
-            self.fig.data[0].y = self.exp_data.loc[:, 'Ψ']
-            self.fig.data[1].y = self.exp_data.loc[:, 'Δ']
+            self.fig.data[0].y = self.exp_data.loc[:, "Ψ"]
+            self.fig.data[1].y = self.exp_data.loc[:, "Δ"]
 
         if update_names:
-            self.fig.data[0].name = 'Ψ'
-            self.fig.data[1].name = 'Δ'
-            self.fig.data[2].name = 'Ψ_tmm'
-            self.fig.data[3].name = 'Δ_tmm'
+            self.fig.data[0].name = "Ψ"
+            self.fig.data[1].name = "Δ"
+            self.fig.data[2].name = "Ψ_tmm"
+            self.fig.data[3].name = "Δ_tmm"
 
     def set_rho(self, update_exp: bool = False, update_names: bool = False) -> None:
         """Sets Plot to Rho values
@@ -62,12 +70,14 @@ class FitRho(FitDecorator):
             self.fig.data[1].y = exp_rho.apply(lambda x: x.imag).values
 
         if update_names:
-            self.fig.data[0].name = 'ρr'
-            self.fig.data[1].name = 'ρi'
-            self.fig.data[2].name = 'ρr_tmm'
-            self.fig.data[3].name = 'ρi_tmm'
+            self.fig.data[0].name = "ρr"
+            self.fig.data[1].name = "ρi"
+            self.fig.data[2].name = "ρr_tmm"
+            self.fig.data[3].name = "ρi_tmm"
 
-    def set_residual(self, update_exp: bool = False, update_names: bool = False) -> None:
+    def set_residual(
+        self, update_exp: bool = False, update_names: bool = False
+    ) -> None:
         """Sets plots to residual values
 
         Args:
@@ -80,18 +90,20 @@ class FitRho(FitDecorator):
         self.fig.update_layout(yaxis_title="Residual")
 
         exp_rho = calc_rho(self.exp_data)
-        self.fig.data[0].y = self.exp_data.loc[:, 'Ψ'] - data.psi
-        self.fig.data[1].y = self.exp_data.loc[:, 'Δ'] - data.delta
+        self.fig.data[0].y = self.exp_data.loc[:, "Ψ"] - data.psi
+        self.fig.data[1].y = self.exp_data.loc[:, "Δ"] - data.delta
         self.fig.data[2].y = exp_rho.apply(lambda x: x.real).values - data.rho.real
         self.fig.data[3].y = exp_rho.apply(lambda x: x.imag).values - data.rho.imag
 
         if update_names:
-            self.fig.data[0].name = 'Psi Res.'
-            self.fig.data[1].name = 'Delta Res.'
-            self.fig.data[2].name = 'ρr Res.'
-            self.fig.data[3].name = 'ρi Res.'
+            self.fig.data[0].name = "Psi Res."
+            self.fig.data[1].name = "Delta Res."
+            self.fig.data[2].name = "ρr Res."
+            self.fig.data[3].name = "ρi Res."
 
-    def set_pseudo_diel(self, update_exp: bool = False, update_names: bool = False) -> None:
+    def set_pseudo_diel(
+        self, update_exp: bool = False, update_names: bool = False
+    ) -> None:
         """Sets Plot to Pseudo Dielectric function values
 
         Args:
@@ -101,22 +113,23 @@ class FitRho(FitDecorator):
                                            Defaults to False.
         """
         data = self.model(self.exp_data.index, self.params)
-        peps = calc_pseudo_diel(pd.DataFrame(data.rho, index=self.exp_data.index).iloc[:, 0],
-                                self.angle)
+        peps = calc_pseudo_diel(
+            pd.DataFrame(data.rho, index=self.exp_data.index).iloc[:, 0], self.angle
+        )
         self.fig.update_layout(yaxis_title="ϵ")
-        self.fig.data[2].y = peps.loc[:, 'ϵ1']
-        self.fig.data[3].y = peps.loc[:, 'ϵ2']
+        self.fig.data[2].y = peps.loc[:, "ϵ1"]
+        self.fig.data[3].y = peps.loc[:, "ϵ2"]
 
         if update_exp:
             exp_peps = calc_pseudo_diel(calc_rho(self.exp_data), self.angle)
-            self.fig.data[0].y = exp_peps.loc[:, 'ϵ1']
-            self.fig.data[1].y = exp_peps.loc[:, 'ϵ2']
+            self.fig.data[0].y = exp_peps.loc[:, "ϵ1"]
+            self.fig.data[1].y = exp_peps.loc[:, "ϵ2"]
 
         if update_names:
-            self.fig.data[0].name = 'ϵ1'
-            self.fig.data[1].name = 'ϵ2'
-            self.fig.data[2].name = 'ϵ1_tmm'
-            self.fig.data[3].name = 'ϵ2_tmm'
+            self.fig.data[0].name = "ϵ1"
+            self.fig.data[1].name = "ϵ2"
+            self.fig.data[2].name = "ϵ1_tmm"
+            self.fig.data[3].name = "ϵ2_tmm"
 
     def update_selection(self, change: dict = None) -> None:
         """Update plot after selection of displayed data
@@ -135,62 +148,78 @@ class FitRho(FitDecorator):
         """Create ipywidgets for parameter estimation"""
         self.param_widgets = {}
         self.selector = widgets.Dropdown(
-            options=['Psi/Delta', 'Rho', 'Pseudo Diel.', 'Residual'],
-            value='Psi/Delta',
-            description='Display: ',
-            disabled=False
+            options=["Psi/Delta", "Rho", "Pseudo Diel.", "Residual"],
+            value="Psi/Delta",
+            description="Display: ",
+            disabled=False,
         )
-        self.selector.observe(self.update_selection, names='value')
+        self.selector.observe(self.update_selection, names="value")
 
         checkboxes = []
         for param in self.params.valuesdict():
-            curr_checkbox = widgets.Checkbox(value=self.params[param].vary,
-                                             description_tooltip=param,
-                                             layout=widgets.Layout(width='15px'),
-                                             indent=False)
-            curr_checkbox.observe(self.set_vary_param, names='value')
-            curr_widget = widgets.BoundedFloatText(self.params[param],
-                                                   min=float_info.min
-                                                    if self.params[param].min == -float('inf')
-                                                    else self.params[param].min,
-                                                   max=float_info.max
-                                                    if self.params[param].max == float('inf')
-                                                    else self.params[param].max,
-                                                   description=param,
-                                                   continuous_update=False)
-            curr_widget.observe(self.update_params, names='value')
+            curr_checkbox = widgets.Checkbox(
+                value=self.params[param].vary,
+                description_tooltip=param,
+                layout=widgets.Layout(width="15px"),
+                indent=False,
+            )
+            curr_checkbox.observe(self.set_vary_param, names="value")
+            curr_widget = widgets.BoundedFloatText(
+                self.params[param],
+                min=float_info.min
+                if self.params[param].min == -float("inf")
+                else self.params[param].min,
+                max=float_info.max
+                if self.params[param].max == float("inf")
+                else self.params[param].max,
+                description=param,
+                continuous_update=False,
+            )
+            curr_widget.observe(self.update_params, names="value")
             checkboxes.append(curr_checkbox)
             self.param_widgets[param] = curr_widget
 
-        fit_button = widgets.Button(description='Fit')
+        fit_button = widgets.Button(description="Fit")
         fit_button.on_click(lambda _: self.fit_button_clicked())
         button_list = [self.selector, fit_button]
 
         if isinstance(self.params, ParamsHist):
-            undo_button = widgets.Button(description='Undo')
+            undo_button = widgets.Button(description="Undo")
             undo_button.on_click(self.re_undo_button_clicked)
-            redo_button = widgets.Button(description='Redo')
+            redo_button = widgets.Button(description="Redo")
             redo_button.on_click(self.re_undo_button_clicked)
-            init_button = widgets.Button(description='Set initial')
+            init_button = widgets.Button(description="Set initial")
             init_button.on_click(lambda _: self.reset_to_init_params())
 
             button_list.append(undo_button)
             button_list.append(redo_button)
             button_list.append(init_button)
 
-        combo_widget = [widgets.HBox([v, w]) for v, w in zip(list(self.param_widgets.values()), checkboxes)]
-        display(widgets.VBox([widgets.HBox(combo_widget +
-                                           button_list,
-                                           layout=widgets.Layout(width='100%',
-                                                                 display='inline-flex',
-                                                                 flex_flow='row wrap')),
-                              self.fig]))
+        combo_widget = [
+            widgets.HBox([v, w])
+            for v, w in zip(list(self.param_widgets.values()), checkboxes)
+        ]
+        display(
+            widgets.VBox(
+                [
+                    widgets.HBox(
+                        combo_widget + button_list,
+                        layout=widgets.Layout(
+                            width="100%", display="inline-flex", flex_flow="row wrap"
+                        ),
+                    ),
+                    self.fig,
+                ]
+            )
+        )
 
-    def fit_function(self,
-                     params: Parameters,
-                     lbda: npt.NDArray,
-                     rhor: npt.NDArray,
-                     rhoi: npt.NDArray) -> npt.NDArray:
+    def fit_function(
+        self,
+        params: Parameters,
+        lbda: npt.NDArray,
+        rhor: npt.NDArray,
+        rhoi: npt.NDArray,
+    ) -> npt.NDArray:
         """The fit function to minimize the fitting problem
 
         Args:
@@ -210,7 +239,7 @@ class FitRho(FitDecorator):
 
         return np.concatenate((resid_rhor, resid_rhoi))
 
-    def fit(self, method='leastsq'):
+    def fit(self, method="leastsq"):
         """Execute lmfit with the current fitting parameters
 
         Args:
@@ -222,10 +251,12 @@ class FitRho(FitDecorator):
             Result: The fitting result
         """
         rho = calc_rho(self.exp_data)
-        res = minimize(self.fit_function,
-                       self.params,
-                       args=(rho.index.to_numpy(), rho.values.real, rho.values.imag),
-                       method=method)
+        res = minimize(
+            self.fit_function,
+            self.params,
+            args=(rho.index.to_numpy(), rho.values.real, rho.values.imag),
+            method=method,
+        )
 
         self.fitted_params = res.params
         return res
@@ -234,34 +265,45 @@ class FitRho(FitDecorator):
         """Plot the fit results as Psi/Delta"""
         fit_result = self.model(self.exp_data.index.to_numpy(), self.fitted_params)
 
-        return go.FigureWidget(pd.concat([self.exp_data,
-                                          pd.DataFrame({'Ψ_fit': fit_result.psi,
-                                                        'Δ_fit': fit_result.delta},
-                                                       index=self.exp_data.index)]
-                                         ).plot())
+        return go.FigureWidget(
+            pd.concat(
+                [
+                    self.exp_data,
+                    pd.DataFrame(
+                        {"Ψ_fit": fit_result.psi, "Δ_fit": fit_result.delta},
+                        index=self.exp_data.index,
+                    ),
+                ]
+            ).plot()
+        )
 
     def plot_rho(self) -> go.Figure:
         """Plot the fit results as Rho"""
         rho = calc_rho(self.exp_data)
         fit_result = self.model(rho.index.to_numpy(), self.fitted_params)
 
-        return go.FigureWidget(pd.DataFrame({'ρr': rho.apply(lambda x: x.real),
-                                             'ρi': rho.apply(lambda x: x.imag),
-                                             'ρcr': fit_result.rho.real,
-                                             'ρci': fit_result.rho.imag},
-                                            index=rho.index).plot())
+        return go.FigureWidget(
+            pd.DataFrame(
+                {
+                    "ρr": rho.apply(lambda x: x.real),
+                    "ρi": rho.apply(lambda x: x.imag),
+                    "ρcr": fit_result.rho.real,
+                    "ρci": fit_result.rho.imag,
+                },
+                index=rho.index,
+            ).plot()
+        )
 
-    def get_model_data(self,
-                       params:Parameters=None,
-                       repr:str='psi-delta',
-                       append_exp_data=False) -> pd.DataFrame:                   
+    def get_model_data(
+        self, params: Parameters = None, repr: str = "psi-delta", append_exp_data=False
+    ) -> pd.DataFrame:
         """Gets the data from the provided modle with the provided parameters.
         If no parameters are provided, the fitted parameters are used
         (which default to the initial parameters if no fit has been triggered).
 
         Args:
-            params (Parameters, optional): 
-                The parameters to calculate the model with. 
+            params (Parameters, optional):
+                The parameters to calculate the model with.
                 If not provided, the fitted parameters are used.
                 Defaults to None.
             repr (str, optional):
@@ -277,36 +319,40 @@ class FitRho(FitDecorator):
         Returns:
             pd.DataFrame: The model results
         """
-        if repr not in ['psi-delta', 'rho']:
-            raise ValueError(f'Unexpected representation: {repr}')
+        if repr not in ["psi-delta", "rho"]:
+            raise ValueError(f"Unexpected representation: {repr}")
 
         if params is None:
             fit_result = self.model(self.exp_data.index.to_numpy(), self.fitted_params)
-            desc = 'fit'
+            desc = "fit"
         else:
             fit_result = self.model(self.exp_data.index.to_numpy(), params)
-            desc = 'model'
+            desc = "model"
 
-        exp_data = {'psi-delta': self.exp_data,
-                    'rho': calc_rho(self.exp_data)}
+        exp_data = {"psi-delta": self.exp_data, "rho": calc_rho(self.exp_data)}
 
-        sim_data = {'psi-delta': pd.DataFrame({f'Ψ_{desc}': fit_result.psi,
-                                               f'Δ_{desc}': fit_result.delta},
-                                               index=self.exp_data.index),
-                    'rho': pd.DataFrame({f'ρ_{desc}': fit_result.rho},
-                                         index=self.exp_data.index)}
+        sim_data = {
+            "psi-delta": pd.DataFrame(
+                {f"Ψ_{desc}": fit_result.psi, f"Δ_{desc}": fit_result.delta},
+                index=self.exp_data.index,
+            ),
+            "rho": pd.DataFrame(
+                {f"ρ_{desc}": fit_result.rho}, index=self.exp_data.index
+            ),
+        }
 
         if append_exp_data:
-            return pd.concat([exp_data[repr],
-                              sim_data[repr]], axis=1)
+            return pd.concat([exp_data[repr], sim_data[repr]], axis=1)
 
         return sim_data[repr]
 
-    def __init__(self,
-                 exp_data: pd.DataFrame,
-                 params: Parameters,
-                 model: Callable[[npt.NDArray, Parameters], Result],
-                 angle: float = 70) -> None:
+    def __init__(
+        self,
+        exp_data: pd.DataFrame,
+        params: Parameters,
+        model: Callable[[npt.NDArray, Parameters], Result],
+        angle: float = 70,
+    ) -> None:
         """Intialize the psi/delta fitting class
 
         Args:
@@ -331,23 +377,35 @@ class FitRho(FitDecorator):
         self.selector = widgets.Dropdown()
         self.last_params = None
         self.initial_params = params.copy()
-        self.fig = go.FigureWidget(pd.concat([exp_data,
-                                              pd.DataFrame({'Ψ_tmm': model(exp_data.index, params).psi,
-                                                            'Δ_tmm': model(exp_data.index, params).delta},
-                                                           index=exp_data.index)]).plot(backend='plotly'))
+        self.fig = go.FigureWidget(
+            pd.concat(
+                [
+                    exp_data,
+                    pd.DataFrame(
+                        {
+                            "Ψ_tmm": model(exp_data.index, params).psi,
+                            "Δ_tmm": model(exp_data.index, params).delta,
+                        },
+                        index=exp_data.index,
+                    ),
+                ]
+            ).plot(backend="plotly")
+        )
         self.fig.update_layout(yaxis_title="Ψ/Δ (°)", xaxis_title="Wavelength (nm)")
 
-        self.update_dict = {'Psi/Delta': self.set_psi_delta,
-                            'Rho': self.set_rho,
-                            'Pseudo Diel.': self.set_pseudo_diel,
-                            'Residual': self.set_residual}
+        self.update_dict = {
+            "Psi/Delta": self.set_psi_delta,
+            "Rho": self.set_rho,
+            "Pseudo Diel.": self.set_pseudo_diel,
+            "Residual": self.set_residual,
+        }
 
         self.create_widgets()
 
 
-def fit(exp_data: pd.DataFrame,
-        params: Parameters,
-        angle: float = 70) -> Callable[[npt.NDArray, Parameters], Result]:
+def fit(
+    exp_data: pd.DataFrame, params: Parameters, angle: float = 70
+) -> Callable[[npt.NDArray, Parameters], Result]:
     """A parameters decorator for fitting psi/delta valus. Displays an ipywidget float box for
     each fitting parameter and an interactive plot to estimate parameters.
 
